function [ mask  scores ] = detectCell( imagename )
%[ mask  scores ] = detectCell( imagename )
%	Developed by Ilya Valmianski
%	ivalmian@ucsd.edu
%
%	detectCell controls a two layer classification process. It is to be given either .tif or .mpd file with raw data and it will 
%	return both the final blob-classifier score (first output) and the intermediate pixel-classifier score (second output)
%	
%	Note, this function is useful to explore in debugger if one wants to undertsand how the classification process is done.

    disp('Initializing..');

    %define where things are
    %ifold='image/';
    bfold='bloblayer/';
    bsfold='blobscore/';
    pfold='scorelayer/';
    psfold='pixelscore/';
    basefold=[pwd '/'];

    stackfile=imagename;%[basefold ifold imagename];
    temp=explode(imagename,'\/');
    imagename=temp{end};
    pixscorefile=[basefold psfold imagename(1:end-2) 'pscore.tif'];
    blobscorefile=[basefold bsfold imagename(1:end-2) 'bscore.tif'];


    disp('Getting pixel score..');
    %get pixel score
    try
        cd(pfold);
        % the arguments are -> image file name, pixel-wise classifier, compute
        % filters
        temp=nan;
        while length(temp)~=1 || ~ischar(temp),
            temp=input('Should we compute the filter images (y/n): ','s');
        end
        compute_filters=1;
        if temp=='y',
            compute_filters=1;
        elseif temp=='n',
            compute_filters=0;
        else
            disp('Cannot understand input, computing filters...');

        end
        [ scores ] = single(scoreIm( stackfile,@firststage,compute_filters ));

        scoretowrite=(scores+20.0)/40*255.0*255.0;
        imwrite(uint16(ceil(scoretowrite)),pixscorefile,'tif');
    catch
        %return to base directory in case of error
        e=lasterror();
        cd(basefold);
        rethrow(e);
    end

    disp('Getting blob score..');
    %get blob score
    try
        cd(['../' bfold]);
        thresh=findthresholds(scores);
        mask=blobDetectScore(pixscorefile,@BlobDetectRobustBoostE6T02S01,thresh);
        maskim=(single(mask)+20)/40.0*255.0*255.0;
        imwrite(uint16(maskim),blobscorefile,'tif');
    catch
        %return to base directory in case of error
        e=lasterror();
        cd(basefold);
        rethrow(e);
    end

    disp('Cleaning up..');
    cd(basefold);
end


function thresh=findthresholds(scores)
    binloc=-20:0.1:20;
    pdfdist=hist(scores(:),binloc);
    pdfdist=pdfdist/sum(pdfdist);
    cdfdist=cumsum(pdfdist);
    
    threshval=0.05:0.05:cdfdist(find(binloc>1,1,'first'));
    thresh=zeros(size(threshval));
    for j=1:length(threshval),
       thresh(j)=binloc(find(cdfdist>threshval(j),1));
    end
end
