//custom header
#include "prefilter.h"

//Matlab entry point
void mexFunction( int nlhs, mxArray *plhs[],
                        int nrhs, const mxArray *prhs[]){
                            
    //verify input
    if(nrhs!=2)
        mexErrMsgTxt("Two inputs are requried by prefilters_opt.cpp");
    if(!mxIsClass(prhs[0],"uint16"))
        mexErrMsgTxt("Input image is expected to be of UINT16 class");
    if(nlhs!=4)
        mexErrMsgTxt("Four outputs are required");
    
    //handle input

    mwSize *inSize= (mwSize *)mxGetDimensions(prhs[0]);    
    UINT16 * inputIm = (UINT16*)(mxGetPr(prhs[0]));                
    double * convSize = (double*)(mxGetPr(prhs[1]));   

    //handle output

    plhs[0]=mxCreateDoubleMatrix(inSize[0],inSize[1],mxREAL);
    plhs[1]=mxCreateDoubleMatrix(inSize[0],inSize[1],mxREAL);
    plhs[2]=mxCreateDoubleMatrix(inSize[0],inSize[1],mxREAL);
    plhs[3]=mxCreateDoubleMatrix(inSize[0],inSize[1],mxREAL);

     
    double * meanFilt=(double *)mxGetData(plhs[0]);
    double * variFilt=(double *)mxGetData(plhs[1]);
    double * covaFilt=(double *)mxGetData(plhs[2]);
    double * corrFilt=(double *)mxGetData(plhs[3]);
    
    if (meanFilt==NULL || variFilt==NULL || covaFilt==NULL || corrFilt==NULL)
        mexErrMsgTxt("could not allocate output");
    
    //do stuff
    
    FILTER *hFilt = new FILTER(inputIm,inSize,convSize);

	int cpySize = inSize[0]*inSize[1];
	memcpy(meanFilt,hFilt->getMean(),cpySize);
	memcpy(variFilt,hFilt->getVari(),cpySize);
	memcpy(covaFilt,hFilt->getCova(),cpySize);
	memcpy(corrFilt,hFilt->getCorr(),cpySize);
	delete [] hFilt;
}