% PrepareBlobTraining(inputdir,outputname)
%	Developed by Ilya Valmianski
%	ivalmian@ucsd.edu
%
%	Given the directory with  matfiles generated by blobchooser (training set) this function will generate 
%	outputname.data and outputname.spec files which should be used as input to your favorite boosting algorithm via jboost
%	This function has no outputs, but if something goes wrong it'll happily throw an error.

function PrepareBlobTraining(inputdir,outputname)
basedir = cd;
cd(inputdir);

%initially this was a script, but converted to function before release
try

	%write spec file
	
	fout=fopen([outputname '.spec'],'w');
    fprintf(fout,['exampleTerminator=;\nattributeTerminator=,\nmaxBadExa=0\n' ...
			    'Threshold number\nArea number\nEulerNumber number\n' ...
				'Extent number\nEccentricity number\nSolidity number\n' ...
				'weight number\nlabels  (1,-1)']);
    fclose(fout);
	
	%load all mat files (assuming they are all training sets) and write them down... 
	files=dir('*.mat');
	for j=1:length(files),

	   fname=files(j).name;
	   disp(['Filename: ' fname]);
	   load(fname);  

	   numthresh=length(obj.threshList);
	   thresh=obj.threshList;
	   
	   score= obj.scoreIm;

	   conElem=zeros([size(score) numthresh]);
	   lindata=[];
	   for k=1:length(obj.boosterData),
	      
	      lindata=[lindata obj.boosterData{k}(:)'];
	   
	      conElem(:,:,k)=bwlabel(score<thresh(k));
	   end
	   
	   %trying to weight negative/positive example  equal.. sometimes jboost is buggy, so you may need to weight simply as 1.
	   pex=sum(lindata==1);
	   nex=sum(lindata==-1);
	   pw=1;
	   nw=pex/nex;
	   
	   for i=1:length(obj.boosterData),%size(conElem,3),
	       
	        prop=regionprops(conElem(:,:,i),'Area',...
	            'EulerNumber','Extent','Eccentricity','Solidity');


	        threshold=thresh(i);
	        conn=conElem(:,:,i); 
	      
	        fout=fopen([outputname '.data'],'a');
	        try
	            for l=1:max(conn(:)),

	                if obj.boosterData{i}(l)==-1,
	                    fprintf(fout,'%f,%f,%f,%f,%f,%f,%f,%d;\n',...
	                        threshold,prop(l).Area,prop(l).EulerNumber,...
	                        prop(l).Extent,prop(l).Eccentricity,prop(l).Solidity,...
	                        nw,-1);
	                elseif obj.boosterData{i}(l)==1,
	                    fprintf(fout,'%f,%f,%f,%f,%f,%f,%f,%d;\n',...
	                        threshold,prop(l).Area,prop(l).EulerNumber,...
	                        prop(l).Extent,prop(l).Eccentricity,prop(l).Solidity,...
	                        pw,+1);
	                end

	            end
	        catch
				e=lasterror();
	            fclose(fout);
	            rethrow(e);
	        end
	        fclose(fout);
	    end
	   
	end
catch
	e=lasterror();
	cd basedir;
	rethrow(e);
end