function [ objData ] = generateObjStruct( stackin )
%GENERATEOBJSTRUCT generates object based on stack
%	Developed by Ilya Valmianski
%	ivalmian@ucsd.edu
    disp('Initializing data structure...');
    objData=struct('Init',1,...         %dataset initialized
                            'meanIm',[],...     %mean image
                            'scoreIm',[],...    %score image
                            'normmeanIm',[],... %normalized mean image
                            'choiceMap',[],...  %choice map image
                            'boosterData',{},...%data as required for learning
                            'objNum',0,...      %current object number
                            'objTot',0,...      %total objects on this level
                            'objSeg',[],...     %ouput of bwlabel on current threshold
                            'threshNum',NaN,... %current theshold
                            'threshList',[]);   %list of applicable threshold

    curdir=pwd;
    disp('Computing pixel score...');
    tic
    try
        cd('./getImages');
    end
    try
		%IMPORTANT!
		% note that calcScoreRSRobustBoost20pEpsilon is the particular pixel classifier, if you want to train based on a different pixel classifier, you need to edit this line
        [scores filters meanim]=scoreIm(stackin,@firststage,1);
    catch e,
        disp(e);
    end
    toc
    cd(curdir);
    disp('Filling in data structure...');
    %ok, fill in the object
    objData(1).meanIm=meanim;%filters(:,:,1);
    objData(1).scoreIm=scores;
    objData(1).normmeanIm=filters(:,:,5);
    
    objData(1).choiceMap=single(repmat(meanim,[1 1 3]));
    objData(1).choiceMap=objData.choiceMap/max(objData.choiceMap(:));
    
    objData(1).threshList=findthresholds(scores);
    objData(1).threshNum=1;
    objData(1).objSeg=bwlabel(scores<objData.threshList(1));
    objData(1).objTot=max(objData.objSeg(:));
    objData(1).objNum=1;
    objData(1).Init=1;
    
    objData(1).boosterData{1}=zeros(objData.objTot,1);
    disp('Done...');

end

function thresh=findthresholds(scores)
    binloc=-20:0.1:20;
    pdfdist=hist(scores(:),binloc);
    pdfdist=pdfdist/sum(pdfdist);
    cdfdist=cumsum(pdfdist);
    
    threshval=0.05:0.05:cdfdist(find(binloc>1,1,'first'));
    thresh=zeros(size(threshval));
    for j=1:length(threshval),
       thresh(j)=binloc(find(cdfdist>threshval(j),1));
    end
end