function mpContent = mp2mat(varargin)
%MP2MAT - reads mp file content and returns data structure
%
%Written by Pablo Blinder (partially based on Phil Tsai's rawdata.m)
% 2008/09/01 - fixed bug with causes error when reading non-square data

%Parse input arguments, if any
fname = varargin{1};

%jd
if exist(fname) == 0
    error 'filename does not exist'
end

%Open stream
f1 = figure(1);
set(f1,'visible','on');

%mpfile = actxcontrol('MPfile.Data');
mpfile = actxcontrol('MPfile.Data',[0,0,500,500],f1);

OpenFile = invoke(mpfile,'OpenMPFile',fname);

%Read header and take further action based on header information
mpContent.Header = mp2mat_readHeader(mpfile);
mpContent.num_frames = str2double(mpContent.Header.Frame_Count);
mpContent.xsize = str2double(mpContent.Header.Frame_Width);
mpContent.ysize = str2double(mpContent.Header.Frame_Height);

for chNum = 1 : 4
    chNumStr = num2str(chNum);
    if strcmp(mpContent.Header.(['Enabled' chNumStr]),'True')
        mpContent.(['Ch' chNumStr]) = mp2mat_getChannelData(mpfile,chNum,mpContent);
    end
end

%jd
close(f1)

%---------------------------------------------------------------------------------------
function chData = mp2mat_getChannelData(mpfile,chNum,mpContent)

h2bar = waitbar(0,['Reading channel' num2str(chNum)]);
%read first data frame and figure out if channel contains analog or image data
tempframe = mpfile.ReadFrameData(chNum,1);
if numel(tempframe) == mpContent.xsize * mpContent.ysize
    %got image
   
    %jd - y size is now first, then x size
    chData = zeros(mpContent.ysize,mpContent.xsize,mpContent.num_frames,'uint16');
    chData(:,:,1) = transpose(reshape(tempframe,mpContent.xsize,mpContent.ysize)); 
    
    for nf = 2 : mpContent.num_frames
        tempframe = mpfile.ReadFrameData(chNum,nf);
        chData(:,:,nf) = transpose(reshape(tempframe,mpContent.xsize,mpContent.ysize));
        waitbar(nf/mpContent.num_frames);
    end
else
    %got analog data
    np = length(tempframe);
    chData = zeros( np *  mpContent.num_frames,1,'int16');
    idx = 1 : np;
    chData(idx) = tempframe;
    for nf = 2 : mpContent.num_frames
        tempframe = mpfile.ReadFrameData(chNum,nf)';
        idx = idx + np;
        chData(idx) = tempframe;
    end
end
close (h2bar);

%---------------------------------------------------------------------------------------
function Header = mp2mat_readHeader(mpfile)
%Read all the header information, but do nothing with it
Header.Scan_Mode = invoke(mpfile,'ReadParameter','Scan Mode');
Header.Frame_Width = invoke(mpfile,'ReadParameter','Frame Width');
Header.Frame_Height = invoke(mpfile,'ReadParameter','Frame Height');
Header.Frame_Count = invoke(mpfile,'ReadParameter','Frame Count');
Header.X_Position = invoke(mpfile,'ReadParameter','X Position');
Header.Y_Position = invoke(mpfile,'ReadParameter','Y Position');
Header.Z_Position = invoke(mpfile,'ReadParameter','Z Position');
Header.Stack_Count = invoke(mpfile,'ReadParameter','Stack Count');
Header.Z_Interval = invoke(mpfile,'ReadParameter','z- Interval');
Header.Averaging_Count = invoke(mpfile,'ReadParameter','Averaging Count');
Header.Repeat_Count = invoke(mpfile,'ReadParameter','Repeat Count');
Header.Magnification = invoke(mpfile,'ReadParameter','Magnification');
Header.Rotation = invoke(mpfile,'ReadParameter','Rotation');
Header.X_Frame_Offset = invoke(mpfile,'ReadParameter','X Frame Offset');
Header.Y_Frame_Offset = invoke(mpfile,'ReadParameter','Y Frame Offset');
Header.Channel_Name1 = invoke(mpfile,'ReadParameter','Channel Name (1)');
Header.Channel_Name2 = invoke(mpfile,'ReadParameter','Channel Name (2)');
Header.Channel_Name3 = invoke(mpfile,'ReadParameter','Channel Name (3)');
Header.Channel_Name4 = invoke(mpfile,'ReadParameter','Channel Name (4)');
Header.Enabled1 = invoke(mpfile,'ReadParameter','Enabled (1)');
Header.Enabled2 = invoke(mpfile,'ReadParameter','Enabled (2)');
Header.Enabled3 = invoke(mpfile,'ReadParameter','Enabled (3)');
Header.Enabled4 = invoke(mpfile,'ReadParameter','Enabled (4)');
Header.Input_Range1 = invoke(mpfile,'ReadParameter','Input Range (1)');
Header.Input_Range2 = invoke(mpfile,'ReadParameter','Input Range (2)');
Header.Input_Range3 = invoke(mpfile,'ReadParameter','Input Range (3)');
Header.Input_Range4 = invoke(mpfile,'ReadParameter','Input Range (4)');
Header.Channel_Unit3 = invoke(mpfile,'ReadParameter','Channel Unit (3)');
Header.Channel_Unit4 = invoke(mpfile,'ReadParameter','Channel Unit (4)');
Header.Channel_Prefix3 = invoke(mpfile,'ReadParameter','Channel Prefix (3)');
Header.Channel_Prefix4 = invoke(mpfile,'ReadParameter','Channel Prefix (4)');
Header.Conversion_Factor3 = invoke(mpfile,'ReadParameter','Conversion Factor (3)');
Header.Conversion_Factor4 = invoke(mpfile,'ReadParameter','Conversion Factor (4)');
Header.Offset3 = invoke(mpfile,'ReadParameter','Offset (3)');
Header.Offset4 = invoke(mpfile,'ReadParameter','Offset (4)');
Header.Data_Point_Per_Frame3 = invoke(mpfile,'ReadParameter','Data Point Per Frame (3)');
Header.Data_Point_Per_Frame4 = invoke(mpfile,'ReadParameter','Data Point Per Frame (4)');
Header.Comments = invoke(mpfile,'ReadParameter','Comments');
