//Standard libs
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

//mex support 
#include "mex.h"

//defines, should be more general later
#define WINDOW_SIDE 20

#define UINT16 unsigned short

void doNormalize(UINT16 *imIn,double *imOut,mwSize *inSize,mwSize *outSize,
                        mwSize *winSize){
                            
    //calculate the actual normalization    
    int curP=0, curPout=0;
    double pixelVal = 0;
    
    double padMean=0;
    for (int j=0; j<inSize[0]; j++){
        for (int k=0; k<inSize[1]; k++){
            curP=j+((k)*inSize[0]);
            padMean+=double(imIn[curP]);
        }
    }
    padMean=padMean/inSize[0]/inSize[1];
    
    for (int j=0; j<inSize[0]; j++){
        for (int k=0; k<inSize[1]; k++){
            
            double pixelMean=0;
            
            //find window mean
            for (int a=-winSize[0]/2; a<winSize[0]/2; a++){
                for (int b=-winSize[1]/2; b<winSize[1]/2; b++){
                    // point corresponding to pixel (a,b) in window
                    // on image imIn
                    curP=(j+a)+((k+b)*inSize[0]); 
                    pixelVal = padMean;
                    if (curP>-1 && curP<(inSize[0]*inSize[1]-1)) 
                        pixelVal=double(imIn[curP]);
                    
                    pixelMean+=pixelVal;
                }
            }
            pixelMean=pixelMean/winSize[0]/winSize[1];
            
            double pixelVar=0;
            
            //find window variance
            for (int a=-winSize[0]/2; a<winSize[0]/2; a++){
                for (int b=-winSize[1]/2; b<winSize[1]/2; b++){
                    // point corresponding to pixel (a,b) in window 
                    // on image imIn
                    curP=(j+a)+((k+b)*inSize[0]); 
                    pixelVal = padMean;
                    if (curP>-1 && curP<(inSize[0]*inSize[1]-1)) 
                        pixelVal=double(imIn[curP]);
                    
                    pixelVar+=(pixelVal-pixelMean)*(pixelVal-pixelMean);
                }
            }
            pixelVar=pixelVar/(winSize[0]*winSize[1]-1);
            
            
            //point corresponding to pixel (j,k)n
            curP=j+((k)*inSize[0]);
            
                     
            imOut[curP] = (double(imIn[curP])-pixelMean)/sqrt(pixelVar);
       
        }
   
        
    }
}                        

//Matlab entry point
void mexFunction( int nlhs, mxArray *plhs[],
                        int nrhs, const mxArray *prhs[]){
                            
    //verify input

    if(nrhs!=1)
        mexErrMsgTxt("A single input is requried by anorm_opt.cpp");
    if(!mxIsClass(prhs[0],"uint16"))
        mexErrMsgTxt("Input expected to be unsigned 16-bit integers");
    if(nlhs!=1)
        mexErrMsgTxt("One output is required");
    
    //intialize

    int imInW = mxGetN(prhs[0]); 
    
    int imInH = mxGetM(prhs[0]);
    
    
    
    mwSize winSize[]={WINDOW_SIDE,WINDOW_SIDE};
    
    mwSize inSize []={imInH,imInW}; 
    mwSize outSize[]={imInH,imInW};
   
    double *outputIm=NULL;
    outputIm=(double*)mxCalloc(outSize[1]*outSize[0],sizeof(double));
    
    UINT16 * inputIm = (UINT16*)(mxGetPr(prhs[0]));
    

    //compute what is needed

    doNormalize(inputIm,outputIm,inSize,outSize,winSize);
    
    //handle output

    plhs[0]=mxCreateDoubleMatrix(imInH,imInW,mxREAL);
    
    if(!plhs[0])
        mexErrMsgTxt("Could not allocate bugger memory\n");
    
    double * outputPr=(double *)mxGetData(plhs[0]);
    int cpySize = imInW*imInH*mxGetElementSize(plhs[0]);
    memcpy(outputPr,outputIm,cpySize);
    mxFree(outputIm);
 
    
}