function pixelTrain( stackName, annotName, runName )

%  pixelTrain( stackName, annotName, runName )
%	Developed by Ilya Valmianski
%	ivalmian@ucsd.edu
%
%	This function  creates a training set based on a *SINGLE* stack and it's annotation. 
%	stackName should be a .tif/.mpd file name while annotName should be an RGB annotation 
%	of that image. The function will generate runName.spec and runName.data files which
%	can act as input for jboost.


savePath='./temp_ims/';

pathElements=explode(stackName,'\/');
filename=pathElements{end};
basename=[savePath filename];
basename=basename(1:(end-4));



hRes=generateImages(stackName,basename);
if ~hRes,
    error('Could not generate images');
end


% all of the filters that we used
iminf=imfinfo([basename '.mean.tif'],'tif');
ims=zeros(iminf(1).Height,iminf(1).Width,8,'uint16');


ims(:,:,1)=medfilt2(imread([basename '.mean.tif']),[5 5]);
ims(:,:,2)=medfilt2(imread([basename '.var.tif']),[5 5]);
ims(:,:,3)=medfilt2(imread([basename '.cov.tif']),[5 5]);
ims(:,:,4)=medfilt2(imread([basename '.corr.tif']),[5 5]);
ims(:,:,5)=medfilt2(imread([basename '.mean.anorm.tif']),[5 5]);
ims(:,:,6)=medfilt2(imread([basename '.var.anorm.tif']),[5 5]);
ims(:,:,7)=medfilt2(imread([basename '.cov.anorm.tif']),[5 5]);
ims(:,:,8)=medfilt2(imread([basename '.corr.anorm.tif']),[5 5]);

annot_raw=imread(annotName);
annot_size=size(annot_raw);
annot_mask=nan(annot_size(1:2));
annot_mask(annot_raw(:,:,2)>200 & annot_raw(:,:,1)<50)=1;
annot_mask(annot_raw(:,:,3)>200 & annot_raw(:,:,1)<50)=0;
annot_mask(isnan(annot_mask))=-1;

[posx posy]=find(annot_mask==1);
[negx negy]=find(annot_mask==-1);
posw=1;
negw=length(posx)/length(negx);
if negw>posw,
    posw=posw/negw;
    negw=1;
end

%generate data file
fout=fopen([runName '.data'],'a');

for t=1:length(posx),
    j=posx(t);
    k=posy(t);
    fprintf(fout,'%f,%f,%f,%f,%f,%f,%f,%f,%f,%d\n',...
        ims(j,k,1),ims(j,k,2),ims(j,k,3),ims(j,k,4),ims(j,k,5),...
        ims(j,k,6),ims(j,k,7),ims(j,k,8),posw,1);

end
for t=1:length(negx),
    j=negx(t);
    k=negy(t);
    fprintf(fout,'%f,%f,%f,%f,%f,%f,%f,%f,%f,%d\n',...
        ims(j,k,1),ims(j,k,2),ims(j,k,3),ims(j,k,4),ims(j,k,5),...
        ims(j,k,6),ims(j,k,7),ims(j,k,8),negw,-1);

end


fclose(fout);

specFile=['exampleTerminator=;\nattributeTerminator=,\nmaxBadExa=0\n'...
    'mean number\nvar number\ncov number\ncorr number\n'...
    'mean.anorm number\nvar.anorm number\ncov.anorm number\n' ....
    'corr.anorm number\nweight number'...
    '\nlabels (1,-1)'];

fs=fopen([runName '.spec'],'w');

fprintf(fs,specFile);

fclose(fs);

